@@ -18,6 +18,7 @@ from HoloAI.HAIBaseConfig.BaseConfig import BaseConfig
 
 load_dotenv()
 
+
 class AnthropicConfig(BaseConfig):
     def __init__(self, apiKey=None):
         super().__init__()
@@ -30,33 +31,52 @@ class AnthropicConfig(BaseConfig):
         if not apiKey:
             raise KeyError("Anthropic API key not found. Please set ANTHROPIC_API_KEY in your environment variables.")
         self.client = anthropic.Anthropic(api_key=apiKey)
-
+        # self.client = OpenAI(
+        #     base_url="https://api.anthropic.com/v1/",
+        #     api_key=apiKey
+        # )
     def _setModels(self):
         self.RModel = os.getenv("ANTHROPIC_TEXT_MODEL", "claude-3-5-haiku-latest")
-        self.VModel = os.getenv("ANTHROPIC_VISION_MODEL", "claude-sonnet-4")
+        self.VModel = os.getenv("ANTHROPIC_VISION_MODEL", "claude-opus-4-20250514")
 
     # ---------------------------------------------------------
     # Response
     # ---------------------------------------------------------
     def Response(self, **kwargs) -> str:
-        keys = [
-            "model", "system", "user", "skills", "tools",
-            "show", "effort", "budget", "tokens", "files", "verbose"
-        ]
+        # keys = [
+        #     "model", "system", "user", "skills", "tools", "choice",
+        #     "show", "effort", "budget", "tokens", "files", "collect", "verbose"
+        # ]
+        keys = self.getKeys("response")
         params = self.extractKwargs(kwargs, keys)
 
         messages = parseJsonInput(params["user"])
 
         args = self._getArgs(params["model"], params["system"], messages, params["tokens"])
 
+        if params["creativity"]:
+            args["temperature"] = params["creativity"]
+
         if params["tools"]:
             args["tools"] = params["tools"]
+            #args["tool_choice"] = params["choice"]
+            args["tool_choice"] = self._mapChoice(params.get("choice"))
 
         if params["skills"]:
             additionalInfo = self.executeSkills(params["skills"], params["user"], params["tokens"], params["verbose"])
             if additionalInfo:
                 messages.append(formatJsonInput("user", additionalInfo))
 
+        # intent = self.extractIntent(params["user"])
+        # fileInfo = extractFileInfo(intent)
+        # if fileInfo:
+        #     messages.append(formatJsonInput("user", fileInfo))
+
+        # files = params["files"]
+        # if files:
+        #     fileInfo = str(extractText(files))
+        #     if fileInfo:
+        #         messages.append(formatJsonInput("user", fileInfo))
         messages += self._DocFiles(params)
 
         if supportsReasoning(params["model"]):
@@ -75,7 +95,7 @@ class AnthropicConfig(BaseConfig):
                 args["max_tokens"] = params["tokens"]
             args["thinking"] = {"type": "enabled", "budget_tokens": params["budget"]}
 
-        response = self._endPoint(**args)
+        response = self._endPoint(**args)# self.client.messages.create(**args)
         if supportsReasoning(params["model"]):
             return response if params["verbose"] else next((block.text for block in response.content if getattr(block, "type", None) == "text"), "")
         return response if params["verbose"] else response.content[0].text
@@ -84,10 +104,11 @@ class AnthropicConfig(BaseConfig):
     # Vision
     # ---------------------------------------------------------
     def Vision(self, **kwargs):
-        keys = [
-            "model", "system", "user", "skills", "tools",
-            "effort", "budget", "tokens", "files", "collect", "verbose"
-        ]
+        # keys = [
+        #     "model", "system", "user", "skills", "tools", "choice",
+        #     "show", "effort", "budget", "tokens", "files", "collect", "verbose"
+        # ]
+        keys = self.getKeys("vision")
         params = self.extractKwargs(kwargs, keys)
 
         images = self._mediaFiles(params["files"], params["collect"])
@@ -105,7 +126,10 @@ class AnthropicConfig(BaseConfig):
 
         args = self._getArgs(params["model"], params["system"], messages, params["tokens"])
 
-        response = self._endPoint(**args)
+        if params["creativity"]:
+            args["temperature"] = params["creativity"]
+        
+        response = self._endPoint(**args)# self.client.messages.create(**args)
         return response if params["verbose"] else response.content[0].text
 
     def processSkills(self, instructions, user, tokens) -> str:
@@ -113,7 +137,7 @@ class AnthropicConfig(BaseConfig):
         intent = self.extractIntent(user)
         messages.append(formatJsonInput("user", intent))
         args = self._getArgs(self.RModel, instructions, messages, tokens)
-        response = self._endPoint(**args)
+        response = self._endPoint(**args)# self.client.messages.create(**args)
         return response.content[0].text
 
     def _endPoint(self, **args):
@@ -128,6 +152,15 @@ class AnthropicConfig(BaseConfig):
         }
         return args
 
+    def _mapChoice(self, choice):
+        mapping = {
+            "auto": "auto",
+            "required": "any",
+            "none": "none"
+        }
+        return mapping[(choice or "auto").lower()]
+
+
     def _mediaFiles(self, files, collect):
         images = []
         for path in files:
