@@ -21,6 +21,7 @@ from HoloAI.HAIBaseConfig.BaseConfig import BaseConfig
 
 load_dotenv()
 
+
 class GoogleConfig(BaseConfig):
     def __init__(self, apiKey=None):
         super().__init__()
@@ -38,14 +39,15 @@ class GoogleConfig(BaseConfig):
         self.RModel = os.getenv("GOOGLE_RESPONSE_MODEL", "gemini-2.5-flash")
         self.VModel = os.getenv("GOOGLE_VISION_MODEL", "gemini-2.5-flash")
 
-    # -----------------------------------------------------------------
+    # ---------------------------------------------------------
     # Response
-    # -----------------------------------------------------------------
+    # --------------------------------------------------------- 
     def Response(self, **kwargs) -> str:
-        keys = [
-            "model", "system", "user", "skills", "tools",
-            "show", "effort", "budget", "tokens", "files", "verbose"
-        ]
+        # keys = [
+        #     "model", "system", "user", "skills", "tools", "choice",
+        #     "show", "effort", "budget", "tokens", "files", "collect", "verbose"
+        # ]
+        keys = self.getKeys("response")
         params = self.extractKwargs(kwargs, keys)
         messages = parseTypedInput(params["user"])
 
@@ -56,12 +58,23 @@ class GoogleConfig(BaseConfig):
             if additionalInfo:
                 messages.append(formatTypedInput("user", additionalInfo))
 
-        messages += self._DocFiles(params)
+        # intent = self.extractIntent(params["user"])
+        # fileInfo = extractFileInfo(intent)
+        # if fileInfo:
+        #     messages.append(formatTypedInput("user", fileInfo))
 
+        # files = params["files"]
+        # if files:
+        #     fileInfo = str(extractText(files))
+        #     if fileInfo:
+        #         messages.append(formatTypedInput("user", fileInfo))
+        messages += self._DocFiles(params)
+        # Use reasoning flag only if model supports it
         reasoning = supportsReasoning(params["model"])
         generateConfig = self._configArgs(params, reasoning=reasoning)
         args = self._getArgs(params["model"], messages, generateConfig)
-
+        # if params["creativity"]:
+        #     args["temperature"] = params["creativity"]
         response = self._endPoint(**args)
         return response if params["verbose"] else response.text
 
@@ -69,10 +82,11 @@ class GoogleConfig(BaseConfig):
     # Vision
     # -----------------------------------------------------------------
     def Vision(self, **kwargs):
-        keys = [
-            "model", "system", "user", "skills", "tools",
-            "effort", "budget", "tokens", "files", "collect", "verbose"
-        ]
+        # keys = [
+        #     "model", "system", "user", "skills", "tools", "choice",
+        #     "show", "effort", "budget", "tokens", "files", "collect", "verbose"
+        # ]
+        keys = self.getKeys("vision")
         params = self.extractKwargs(kwargs, keys)
 
         images = self._mediaFiles(params["files"], params["collect"])
@@ -80,9 +94,10 @@ class GoogleConfig(BaseConfig):
         textPart = types.Part(text=params["user"])
         messages = [types.Content(role="user", parts=images + [textPart])]
 
+        # Vision typically doesn't use reasoning configspass reasoning=False
         generateConfig = self._configArgs(params, reasoning=False)
         args = self._getArgs(params["model"], messages, generateConfig)
-
+        
         response = self._endPoint(**args)
         return response if params["verbose"] else response.text
 
@@ -92,31 +107,66 @@ class GoogleConfig(BaseConfig):
         params = {
             "system": instructions,
             "tokens": tokens,
+            # Not needed for skills: model/tools/effort/budget
         }
+        # Always pass reasoning=False here
         generateConfig = self._configArgs(params, reasoning=False)
         args = self._getArgs(self.RModel, messages, generateConfig)
         response = self._endPoint(**args)
         return response.text
 
+
     def _endPoint(self, **args) -> str:
         return self.client.models.generate_content(**args)
 
+    # def _configArgs(self, params, reasoning=False):
+    #     configArgs = {
+    #         "response_mime_type": "text/plain",
+    #         "system_instruction": [params["system"]],
+    #         "max_output_tokens": params["tokens"],
+    #     }
+    #     if params.get("tools"):
+    #         configArgs["tools"] = params["tools"]
+
+    #     if reasoning and supportsReasoning(params["model"]):
+    #         if params["effort"] == "auto":
+    #             params["budget"] = -1  # Or whatever logic you need
+    #         configArgs["thinking_config"] = types.ThinkingConfig(thinking_budget=params["budget"])
+
+    #     return types.GenerateContentConfig(**configArgs)
+
     def _configArgs(self, params, reasoning=False):
         configArgs = {
             "response_mime_type": "text/plain",
             "system_instruction": [params["system"]],
             "max_output_tokens": params["tokens"],
         }
+        if params.get("creativity"):
+            configArgs["temperature"] = params["creativity"]
+
         if params.get("tools"):
             configArgs["tools"] = params["tools"]
+            # Map the normalized choice param for Google GenAI SDK
+            configArgs["function_calling_config"] = types.FunctionCallingConfig(
+                mode=self._mapChoice(params.get("choice"))
+            )
 
         if reasoning and supportsReasoning(params["model"]):
             if params["effort"] == "auto":
                 params["budget"] = -1  # Or whatever logic you need
             configArgs["thinking_config"] = types.ThinkingConfig(thinking_budget=params["budget"])
-
+        
         return types.GenerateContentConfig(**configArgs)
 
+    def _mapChoice(self, choice):
+        # Always work in lower-case for safety
+        mapping = {
+            "auto": "AUTO",        # Let model decide
+            "required": "ANY",     # Force a tool call (Google uses "ANY" for this)
+            "none": "NONE"         # Disable function calling
+        }
+        return mapping.get((choice or "auto").lower(), "AUTO")
+
     def _getArgs(self, model, messages, config):
         args = {
             "model": model,
