@@ -21,6 +21,7 @@ from HoloAI.HAIBaseConfig.BaseConfig import BaseConfig
 
 load_dotenv()
 
+
 class OpenAIConfig(BaseConfig):
     def __init__(self, apiKey=None):
         super().__init__()
@@ -35,17 +36,18 @@ class OpenAIConfig(BaseConfig):
         self.client = OpenAI(api_key=apiKey)
 
     def _setModels(self):
-        self.RModel = os.getenv("OPENAI_RESPONSE_MODEL", "gpt-4.1-mini")
+        self.RModel = os.getenv("OPENAI_RESPONSE_MODEL", "gpt-4.1")
         self.VModel = os.getenv("OPENAI_VISION_MODEL", "gpt-4.1")
 
     # ---------------------------------------------------------
     # Response
     # ---------------------------------------------------------
     def Response(self, **kwargs) -> str:
-        keys = [
-            "model", "system", "user", "skills", "tools",
-            "show", "effort", "budget", "tokens", "files", "verbose"
-        ]
+        # keys = [
+        #     "model", "system", "user", "skills", "tools", "choice",
+        #     "show", "effort", "budget", "tokens", "files", "collect", "verbose"
+        # ]
+        keys = self.getKeys("response")
         params = self.extractKwargs(kwargs, keys)
         # Now use params["model"], params["system"], etc.
         messages = []
@@ -53,14 +55,30 @@ class OpenAIConfig(BaseConfig):
         messages.extend(parseJsonInput(params["user"]))
         args = self._getArgs(params["model"], messages, params["tokens"])
 
+        if params["creativity"]:
+            args["temperature"] = params["creativity"]
+            #print(f"Setting temperature to {params['creativity']}")
+
         if params["tools"]:
             args["tools"] = params["tools"]
+            # #args["tool_choice"] = params["choice"]
+            args["tool_choice"] = self._mapChoice(params.get("choice"))
 
         if params["skills"]:
             additionalInfo = self.executeSkills(params["skills"], params["user"], params["tokens"], params["verbose"])
             if additionalInfo:
                 messages.append(formatJsonInput("user", additionalInfo))
 
+        # intent = self.extractIntent(params["user"])
+        # fileInfo = extractFileInfo(intent)
+        # if fileInfo:
+        #     messages.append(formatJsonInput("user", fileInfo))
+
+        # files = params["files"]
+        # if files:
+        #     fileInfo = str(extractText(files))
+        #     if fileInfo:
+        #         messages.append(formatJsonInput("user", fileInfo))
         messages += self._DocFiles(params)
 
         if supportsReasoning(params["model"]):
@@ -70,17 +88,18 @@ class OpenAIConfig(BaseConfig):
             if params["budget"]:
                 args["max_output_tokens"] = params["budget"]
 
-        response = self._endPoint(**args)
+        response = self._endPoint(**args)# self.client.responses.create(**args)
         return response if params["verbose"] else response.output_text
 
     # ---------------------------------------------------------
     # Vision
     # ---------------------------------------------------------
     def Vision(self, **kwargs):
-        keys = [
-            "model", "system", "user", "skills", "tools",
-            "effort", "budget", "tokens", "files", "collect", "verbose"
-        ]
+        # keys = [
+        #     "model", "system", "user", "skills", "tools", "choice",
+        #     "show", "effort", "budget", "tokens", "files", "collect", "verbose"
+        # ]
+        keys = self.getKeys("vision")
         params = self.extractKwargs(kwargs, keys)
         messages = []
         messages.append(formatJsonInput("system", params["system"]))
@@ -95,7 +114,11 @@ class OpenAIConfig(BaseConfig):
         })
 
         args = self._getArgs(params["model"], payload, params["tokens"])
-        response = self._endPoint(**args)
+
+        if params["creativity"]:
+            args["temperature"] = params["creativity"]
+        
+        response = self._endPoint(**args)# self.client.responses.create(**args)
         return response if params["verbose"] else response.output_text
 
     def processSkills(self, instructions, user, tokens) -> str:
@@ -104,7 +127,7 @@ class OpenAIConfig(BaseConfig):
         intent = self.extractIntent(user)
         messages.append(formatJsonInput("user", intent))
         args = self._getArgs(self.RModel, messages, tokens)
-        response = self._endPoint(**args)
+        response = self._endPoint(**args)# self.client.responses.create(**args).output_text
         return response.output_text
 
     def _endPoint(self, **args) -> str:
@@ -118,6 +141,15 @@ class OpenAIConfig(BaseConfig):
         }
         return args
 
+    def _mapChoice(self, choice):
+        mapping = {
+            "auto": "auto",
+            "required": "required",
+            "none": "none"
+        }
+        return mapping[(choice or "auto").lower()]
+
+
     def _mediaFiles(self, files, collect):
         images = []
         for path in files:
@@ -140,4 +172,4 @@ class OpenAIConfig(BaseConfig):
             fileInfo = str(extractText(files))
             if fileInfo:
                 messages.append(formatJsonInput("user", fileInfo))
-        return messages
\ No newline at end of file
+        return messages
