@@ -151,22 +151,21 @@ class HoloAI:
             return
 
         self.providerMap = setProvider()
-
         self.initialized = True
 
     def getFrameworkInfo(self):
-        """
-        Returns a string with framework information.
-        """
         return getFrameworkInfo()
 
     def listProviders(self):
-        """
-        Returns a list of available model providers.
-        This is based on the keys of the providerMap dictionary.
-        """
         return list(self.providerMap.keys())
 
+    def _inferModelProvider(self, model: str):
+        return next(
+            (provider for prefixes, provider in MODELS.items()
+             if any(model.startswith(prefix) for prefix in prefixes)),
+            None
+        )
+
     def setProvider(self, apiInput=None):
         """
         Sets provider API keys from:
@@ -179,21 +178,12 @@ class HoloAI:
         """
         self.providerMap = setProvider(apiInput)
 
-    def _inferModelProvider(self, model: str):
-        """
-        Infers the provider based on the model name.
-        Returns the provider name as a string, or None if not found.
-        """
-        return next(
-            (provider for prefixes, provider in MODELS.items()
-             if any(model.startswith(prefix) for prefix in prefixes)),
-            None
-        )
-
     def _getProviderConfig(self, model: str):
         """
-        Returns the config instance strictly based on model's inferred provider.
-        Raises if provider cannot be inferred.
+        Returns the provider configuration for a given model.
+        :param model: (str) The model name to infer the provider from.
+        :return: The provider configuration object.
+        :raises ValueError: If the provider cannot be inferred from the model.
         """
         provider = self._inferModelProvider(model)
         if provider and provider in self.providerMap:
@@ -214,9 +204,21 @@ class HoloAI:
             - user/input: (str or list) The main user input (Required).
                 Accepts a single prompt string or a message history (list of messages).
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
-            - skills: (list) Skills to use (Optional).
-            - tools: (list) Tools to use (Optional).
+            - skills: Skills and Actions to use (Optional).
+                Skills and actions will be executed if the model chooses to call them.
+                Valid options:
+                    - skills=[skills, actions]
+                    - skills=skills, 
+                      actions=actions
+                If only actions is provided (without skills), this is not allowed and will result in an error.
+            - tools: (list) Tools to use (Optional) [tools].
+            - toolCoice/tool_choice: (str) Controls model tool-calling behavior.
+                Valid values:
+                    - 'auto': Model decides when to call tools (default)
+                    - 'required': Model must call a tool (forces function/tool call)
+                    - 'none': Tool/function calling is disabled
             - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
+            - creativity/temperature: (float) Creativity level (0.0 to 1.0, default: 0.5).
             - effort: (str) Effort level ('auto', 'low', 'medium', 'high') (Optional (default: 'auto')).
             - budget/max_budget: (int) Budget for the response (Optional (default: 1369)).
             - files: (list) List of file paths can be past in manually or during runtime (default: empty list).
@@ -239,9 +241,21 @@ class HoloAI:
             - user/input: (str or list) The main user input (Required).
                 Accepts a single prompt string or a message history (list of messages).
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
-            - skills: (list) Skills to use (Optional).
-            - tools: (list) Tools to use (Optional).
+            - skills: Skills and Actions to use (Optional).
+                Skills and actions will be executed if the model chooses to call them.
+                Valid options:
+                    - skills=[skills, actions]
+                    - skills=skills, 
+                      actions=actions
+                If only actions is provided (without skills), this is not allowed and will result in an error.
+            - tools: (list) Tools to use (Optional) [tools].
+            - toolCoice/tool_choice: (str) Controls model tool-calling behavior.
+                Valid values:
+                    - 'auto': Model decides when to call tools (default)
+                    - 'required': Model must call a tool (forces function/tool call)
+                    - 'none': Tool/function calling is disabled
             - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
+            - creativity/temperature: (float) Creativity level (0.0 to 1.0, default: 0.5).
             - effort: (str) Effort level ('auto', 'low', 'medium', 'high') (Optional (default: 'auto')).
             - budget/max_budget: (int) Budget for the response (Optional (default: 1369)).
             - files: (list) List of file paths can be past in manually or during runtime (default: empty list).
@@ -259,8 +273,14 @@ class HoloAI:
             - user/input: (str or list) The main user input (Required). 
                 Accepts a single prompt string or a message history (list of messages). 
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
-            - tools: (list) Tools to use (Optional).
+            - tools: (list) Tools to use (Optional) [tools].
+            - toolCoice/tool_choice: (str) Controls model tool-calling behavior.
+                Valid values:
+                    - 'auto': Model decides when to call tools (default)
+                    - 'required': Model must call a tool (forces function/tool call)
+                    - 'none': Tool/function calling is disabled
             - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
+            - creativity/temperature: (float) Creativity level (0.0 to 1.0, default: 0.5).
             - effort: (str) Effort level ('auto', 'low', 'medium', 'high') (Optional (default: 'auto')).
             - budget/max_budget: (int) Budget for the response (Optional (default: 1369)).
             - files: (list) List of file paths can be past in manually or during runtime (default: empty list).
@@ -278,8 +298,14 @@ class HoloAI:
             - user/input: (str or list) The main user input (Required). 
                 Accepts a single prompt string or a message history (list of messages). 
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
-            - tools: (list) Tools to use (Optional).
+            - tools: (list) Tools to use (Optional) [tools].
+            - toolCoice/tool_choice: (str) Controls model tool-calling behavior.
+                Valid values:
+                    - 'auto': Model decides when to call tools (default)
+                    - 'required': Model must call a tool (forces function/tool call)
+                    - 'none': Tool/function calling is disabled
             - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
+            - creativity/temperature: (float) Creativity level (0.0 to 1.0, default: 0.5).
             - files: (list) List of file paths can be past in manually or during runtime (default: empty list).
             - verbose: (bool) Return verbose output (Optional (default: False)).
         :return: A Response object.
@@ -296,6 +322,7 @@ class HoloAI:
                 Accepts a single prompt string or a message history (list of messages). 
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
             - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
+            - creativity/temperature: (float) Creativity level (0.0 to 1.0, default: 0.5).
             - files: (list) List of image file paths (default: empty list).
             - collect: (int) Number of frames to collect (default: 10).
             - verbose: (bool) Return verbose output (Optional (default: False)).
@@ -313,8 +340,14 @@ class HoloAI:
             - user/input: (str or list) The main user input (Required). 
                 Accepts a single prompt string or a message history (list of messages). 
                 Both 'user' and 'input' are interchangeable; use either (Preferred: input).
-            - tools: (list) Tools to use (Optional).
+            - tools: (list) Tools to use (Optional) [tools].
+            - toolCoice/tool_choice: (str) Controls model tool-calling behavior.
+                Valid values:
+                    - 'auto': Model decides when to call tools (default)
+                    - 'required': Model must call a tool (forces function/tool call)
+                    - 'none': Tool/function calling is disabled
             - tokens/max_tokens: (int) Max tokens to use (Optional (default: 3369)).
+            - creativity/temperature: (float) Creativity level (0.0 to 1.0, default: 0.5).
             - effort: (str) Effort level ('auto', 'low', 'medium', 'high') (Optional (default: 'auto')).
             - budget/max_budget: (int) Budget for the response (Optional (default: 1369)).
             - files: (list) List of file paths can be past in manually or during runtime (default: empty list).
@@ -322,7 +355,6 @@ class HoloAI:
             - verbose: (bool) Return verbose output (Optional (default: False)).
         :return: A Reasoning, Response, or Vision object.
         """
-        #return self._routeResponse(**kwargs)
         task = kwargs.get('task', 'response').lower()
         taskMap = {
             'reasoning': self._routeResponse,
@@ -382,14 +414,12 @@ class HoloAI:
         return modeMap[mode]()
 
     def _routeResponse(self, **kwargs):
-        #print(f"\n[Response Request] {kwargs}")
         kwargs = {k.lower(): v for k, v in kwargs.items()}
         model  = kwargs.get('model')
         config = self._getProviderConfig(model)
         return config.getResponse(**kwargs)
 
     def _routeVision(self, **kwargs):
-        #print(f"\n[Vision Request] {kwargs}")
         kwargs = {k.lower(): v for k, v in kwargs.items()}
         model  = kwargs.get('model')
         config = self._getProviderConfig(model)
